local aabb = require 'aabb'

local A = {l=-400,t=-180,w=100,h=100,  dx=200, dy=30}
local B = {l=-100,t=-50,w=50,h=200, dx=-100, dy=-40}

local MODES = { 'A', 'DA', 'B', 'DB' }
local modeIndex = 1

local function drawRect(l,t,w,h, r,g,b,a)
  a = a or 255
  love.graphics.setColor(r,g,b,a)
  love.graphics.rectangle('line', l,t,w,h)
end

local function drawBox(box, r,g,b)
  drawRect(box.l, box.t, box.w, box.h, r,g,b)

  love.graphics.setColor(r,g,b,255)
  local cx, cy = aabb.getCenter(box.l, box.t, box.w, box.h)
  love.graphics.line(cx, cy, cx + box.dx, cy + box.dy)
end

local function drawDebugInfo(t)
  local ml,mt,mw,mh = aabb.getMinkowskyDiff(A.l,A.t,A.w,A.h, B.l,B.t,B.w,B.h)
  local dx,dy = A.dx-B.dx, A.dy-B.dy
  drawRect(ml,mt,mw,mh, 0,255,255)
  love.graphics.line(0,0,dx,dy)

  love.graphics.setColor(255,255,255)
  love.graphics.line(0,0, t*dx, t*dy)
  love.graphics.circle('line', t*dx, t*dy, 5)
end



function love.draw()

  love.graphics.push()
  love.graphics.translate(400,300)

  local mx1,my1,mx2,my2,t = aabb.getDisplacement(A.l,A.t,A.w,A.h, B.l,B.t,B.w,B.h, A.dx,A.dy, B.dx,B.dy)
  if not mx1 then
    mx1,my1,mx2,my2,t = A.dx,A.dy,B.dx,B.dy,0
  end

  drawBox(A, 255,0,0)
  drawBox(B, 0,255,0)

  drawRect(A.l + mx1, A.t + my1, A.w, A.h, 255, 0, 0, 70)
  drawRect(B.l + mx2, B.t + my2, B.w, B.h, 0, 255, 0, 70)

  drawDebugInfo(t)

  love.graphics.setColor(255,255,255)
  love.graphics.circle('fill', 0,0, 5)

  love.graphics.pop()
end

function love.update()
  local mode = MODES[modeIndex]
  local mx, my = love.mouse.getPosition()
  mx, my = mx -400, my-300
  if mode == 'A' then
    A.l, A.t = mx - A.w * .5, my - A.h * .5
  elseif mode == 'B' then
    B.l, B.t = mx - B.w * .5, my - B.h * .5
  elseif mode == 'DA' then
    A.dx, A.dy = mx - (A.l + A.w * .5), my - (A.t + A.h * .5)
  else
    B.dx, B.dy = mx - (B.l + B.w * .5), my - (B.t + B.h * .5)
  end
end

function love.mousepressed()
  modeIndex = (modeIndex + 1) % #MODES
end

function love.keypressed(key)
  if key == 'escape' then love.event.quit() end
end
